@@ -9,7 +9,7 @@ import time
 import tempfile
 import pprint
 import subprocess
-import ruamel.yaml as yaml
+import yaml
 import argparse
 import json
 import base64
@@ -83,7 +83,7 @@ def parse_nbline(line):
         try:
             value=literal_eval(value_str.strip())
             python_type = type(value)
-        except Exception as e:
+        except Exception:
             value = value_str
             python_type = str
 
@@ -158,8 +158,7 @@ class NotebookAdapter:
     def new_tmpdir(self):
         logger.debug("tmpdir was "+getattr(self,'_tmpdir','unset'))
         self._tmpdir = None
-        new_tmpdir = self.tmpdir
-        logger.debug("tmpdir became "+self._tmpdir)
+        logger.debug("tmpdir became %s", self._tmpdir)
 
         return self.tmpdir
 
@@ -298,9 +297,10 @@ class NotebookAdapter:
             logger.info("new tmpdir: %s", tmpdir)
 
             try:
-                logger.info(subprocess.check_output(["git","clone",os.path.dirname(os.path.realpath(self.notebook_fn)), tmpdir]))
-            except:
-                logger.info("git clone failed, will attempt copytree")
+                output = subprocess.check_output(["git","clone",os.path.dirname(os.path.realpath(self.notebook_fn)), tmpdir])
+                logger.info("git clone output: %s", output)
+            except Exception as e:
+                logger.warning("git clone failed: %s, will attempt copytree", e)
 
                 os.rmdir(tmpdir)
 
@@ -421,12 +421,37 @@ except Exception as e:
             output_gather_content+="\nisinstance({output},str) and os.path.exists({output}) and sb.glue(\"{output}_content\",base64.b64encode(open({output},'rb').read()).decode())".format(output=output)
             output_gather_content+="\n".format(output=output)
 
+        nb = self.read()
+
         newcell = nbformat.v4.new_code_cell(source=output_gather_content)
         newcell.metadata['tags'] = ['injected-gather-outputs']
 
-        nb=self.read()
+        nb = self.read()
+
+        if nbformat.current_nbformat !=  nb.nbformat:
+            logger.error("we assume nbformat version %s, but provided notebook is version %s, refusing!", 
+                          nbformat.current_nbformat,
+                          nb.nbformat)
+            raise RuntimeError("incompatabile notebook major version")
+
+        logger.info("provided notebook nbformat version minor %s while nbformat package minor version %s",
+                    nb.nbformat_minor, nbformat.current_nbformat_minor)
+                
+        if nbformat.current_nbformat_minor == nb.nbformat_minor:
+            logger.info("versions of notebook and environment match")
+        elif nbformat.current_nbformat_minor < nb.nbformat_minor:
+            logger.warn("notebook is newer than envionment package! please update your system or expect warnings")
+        elif  nbformat.current_nbformat_minor > nb.nbformat_minor:
+            logger.warning("will attempt to convert, but expect other warnings!")                            
+
+            nb = nbformat.v4.convert.upgrade(nb, from_minor=nb.nbformat_minor)
+        else:
+            raise NotImplementedError
+
+
         nb.cells = nb.cells + [newcell] 
 
+        logger.info("stored pre-processed notebook as %s", self.preproc_notebook_fn)
         pm.iorw.write_ipynb(nb, self.preproc_notebook_fn)
 
     def get_system_parameter_value(self, name, default):
@@ -502,15 +527,12 @@ def nbreduce(nb_source, max_size_mb):
                 largest_cellsize = cellsize
 
             logging.info("cell size %.5lg largest cell size %.5lg", cellsize, largest_cellsize)
-            #logging.info("cell size %.5lg largest cell size %.5lg; metadata %s", cellsize, largest_cellsize, str(cell.metadata))
-
+            
             if 'injected-gather-outputs' in cell.metadata.get('tags', []):
                 logging.info("is injected-gather-outputs: skipping")
                 continue
 
             if cellsize_limit is not None and cellsize >= cellsize_limit:
-                #newcell = nbformat.v4.new_code_cell(source="# redacted")
-                #newcells.append(newcell)
                 logging.info('cleaning cell')
                 cell.outputs = []
                 largest_cellsize = None
@@ -535,7 +557,11 @@ def nbreduce(nb_source, max_size_mb):
             logging.info('notebook size %.4lg Mb is smaller than required %.5lg Mb, only cleaning gathering', current_size_mb, max_size_mb)
             return
         else:
-            logging.info('notebook size %.4lg Mb is larger than required %.5lg Mb, setting cell size limit to the largest cell %.5lg', current_size_mb, max_size_mb, cellsize_limit)
+            logging.info('notebook size %.4lg Mb is larger than required %.5lg Mb, setting cell size limit to the largest cell %s', 
+                         current_size_mb, 
+                         max_size_mb, 
+                         cellsize_limit)
+
             cellsize_limit = largest_cellsize
 
 
@@ -570,7 +596,7 @@ def nbrun(nb_source, inp, inplace=False):
         fn = nba.export_html()
         open("{}_output.ipynb".format(nba.name), "wb").write(open(nba.output_notebook_fn, "rb").read())
 
-        raise Exception("FAILED to execute {}: {}".format(nba.name, exceptions))
+        raise Exception(f"FAILED to execute {nba.name} {exceptions} html exported in {fn}")
 
     r={}
     for k,v in nba.extract_output().items():
@@ -585,11 +611,11 @@ def nbrun(nb_source, inp, inplace=False):
     nbdata = open(nba.output_notebook_fn, "rb").read()
     nbfn = "{}_output.ipynb".format(nba.name)
     open(nbfn, "wb").write(nbdata)
-        
 
+    #TODO: store if not too big?   
     #r['output_notebook'] = nbfn
     #r['output_notebook_content'] = base64.b64encode(nbdata).decode()
-    
+        
     htmlfn = "{}_output.html".format(nba.name)
     nba.export_html(htmlfn)
     
